@@ -2,7 +2,7 @@
 # written by Ralf Biehl at the Forschungszentrum Jülich ,
 # Jülich Center for Neutron Science 1 and Institute of Complex Systems 1
 #    Jscatter is a program to read, analyse and plot data
-#    Copyright (C) 2015-2019  Ralf Biehl
+#    Copyright (C) 2015-2021  Ralf Biehl
 #
 #    This program is free software: you can redistribute it and/or modify
 #    it under the terms of the GNU General Public License as published by
@@ -74,6 +74,7 @@ import os
 import sys
 import time
 
+import shutil
 import subprocess
 from math import pi
 
@@ -86,26 +87,12 @@ from .dataarray import dataList as dL
 
 # As CONTIN uses cgs units we use these here too
 
-kb = 1.3806505e-16  # cm2 g s-2 K-1
+kb = 1.3806505e-16  # cm2 g s-2 K-1 = erg/K =1e6 J/K
 
 path = os.path.realpath(os.path.dirname(__file__))
 
-# http://bugs.pythoorg/issue1731717
-# this workaround at http://bugs.pythoorg/issue1236
-# subprocess._cleanup = lambda: None
-subprocess._cleanup = lambda: None
-
 # find continexe
-# noinspection PyBroadException
-try:
-    p = subprocess.Popen('command -v contin', shell=True,
-                         bufsize=0,
-                         stdin=subprocess.PIPE,
-                         stdout=subprocess.PIPE,
-                         stderr=subprocess.PIPE)
-    continexe = p.communicate()[0].strip()
-except:
-    continexe = ''
+continexe = shutil.which('contin')
 
 
 def _getfirst(a):
@@ -261,45 +248,51 @@ def contin(datalist, Ngrid=256, tmin=-2, tmax=-2, bgr=0, distribution='dls', RDG
     Returns
     -------
         dataList : input with best solution added as attributes prepended by 'contin_'
-         - .contin_bestFit               **best fit result**
-         - .contin_result_fit            fitted correlation function
-         - .contin_fits                  sequence of "not best fits" in CONTIN with same structure as best Fit
-         - .contin_alpha                 ALPHA of best solution
-         - .contin_alphalist             list of ALPHA in fits
+         - .contin_bestFit :              **best fit result**
+         - .contin_result_fit :           fitted correlation function
+         - .contin_fits :                 sequence of "not best fits" in CONTIN with same structure as best Fit
+         - .contin_alpha :                ALPHA of best solution
+         - .contin_alphalist :            list of ALPHA in fits
 
         bestFit dataArray with best solution distribution content
          - .contin_bestFit contains
               [*relaxationtimes, intensityweightci, errors, hydrodynamicradii, massweightci, numberweightci*]
+
+              The columns can be adressed with underscore as e.g. *.contin_bestFit._hydrodynamicradii*.
+              See second example.
+
            Weighted ci are normalized that the *sum* over all points is equal 1.
-           The correponding probability is the probability of the intervall around the corresponding
+           The corresponding probability is the probability of the interval around the corresponding
            relaxation time or Rh. Remember that because of the log scale the interval change width.
 
          - .contin_bestFit.attr()      shows all available parameters from CONTIN fit and other parameters
          - .contin_bestFit.fitquality  measure for the fit quality ALPHA and ...
-         - .contin_bestFit.peaks       original CONTIN results as intensity weight relaxation time distribution
-         - .contin_bestFit.ipeaks      data extracted from the peaks as list for all peaks in a solution
-           it is a intensity weight relaxation time distribution with content
-            - [weight                 peak weight
-            -   mean                  peak mean
-            -   std                   peak standard deviation
-            -   mean_err              error of mean
-            -   imean                 index of peak mean
-            -   1/(q**2*mean)         diffusion constant in cm**2/s
-            -   Rh                    hydrodynamic radius in cm
-            -   q                     wavevector in 1/cm
-            -   1/(q**2*mean)         diffusion constant in nm**2/ns
-            -   Rh                    hydrodynamic radius in nm
-            -   q]                    wavevector in 1/nm
 
-         - .contin_bestFit.mpeaks      same  for 2 strongest peaks mass  weight
-         - .contin_bestFit.npeaks      same  for 2 strongest peaks number weight
-         - .contin_bestFit.ipeaks_name content of ipeaks
-         - .contin_bestFit.info'
-         - .contin_bestFit.baseline'   baseline + error
-         - .contin_bestFit.momentEntireSolution'   all peaks together
-         - .contin_bestFit.maximum'    maximum value
-         - .contin_bestFit.filename'
-         - .contin_bestFit.imaximum'   index maximum value
+        Peak results from CONTIN :
+
+         - .contin_bestFit.peaks       **intensity weight relaxation time distribution** as dict, primary result
+         - .contin_bestFit.ipeaks      **intensity weight relaxation time distribution** for all peaks with :
+            - [weight :                peak weight
+            -   mean  :                peak mean
+            -   std   :                peak standard deviation
+            -   mean_err :             error of mean
+            -   imean :                index of peak mean
+            -   1/(q**2*mean) :        diffusion constant in cm**2/s
+            -   Rh  :                  hydrodynamic radius in cm
+            -   q   :                  wavevector in 1/cm
+            -   1/(q**2*mean) :        diffusion constant in nm**2/ns
+            -   Rh :                   hydrodynamic radius in nm
+            -   q] :                   wavevector in 1/nm
+
+         - .contin_bestFit.mpeaks :     **peaks mass weight** same content as ipeaks for 2 strongest
+         - .contin_bestFit.npeaks :     **peaks number weight** same content as ipeaks for 2 strongest
+         - .contin_bestFit.ipeaks_name : content of ipeaks
+         - .contin_bestFit.info :
+         - .contin_bestFit.baseline :  baseline + error
+         - .contin_bestFit.momentEntireSolution :  all peaks together
+         - .contin_bestFit.maximum :   maximum value
+         - .contin_bestFit.filename :
+         - .contin_bestFit.imaximum :  index maximum value
      
     Notes
     -----
@@ -325,24 +318,27 @@ def contin(datalist, Ngrid=256, tmin=-2, tmax=-2, bgr=0, distribution='dls', RDG
 
     **An example dataset with noise**
     In this synthetic example the noise causes a peak width of about 20% (std/mean) as it is
-    typically found for monodisperse samples (e.g. proteins).
+    typically found for monodisperse samples measured short times (e.g. proteins on a Zetasizer in automatic mode).
     This contribution is not caused by polydispersity and always present dependent on noise level.
+    Noise level can be reduced using longer measurement times, thus reducing the width.
+    In real measurements one needs to test with different measurement times if polydispersity is present
+    (thus independent on noise level).
 
-    D = 0.05 nm²/ns corresponds to a hydrodynamic radius of 4.3 nm and results in a
-    relaxation time :math:`\Gamma= q^2D \approx 90.2 \mu s`.
-
-     - The below plots shows the CONTIN result as correlation time distribution
-       with mean 92 µs ±17 µs and peak weight 95% in good approximation.
-     - The second plot shows the radius distrubution for different weights.
-       Probabilities are for an interval around a point x value that the sum of all points is normalized.
-       (not the integral on the log scale).
+    - D = 0.05 nm²/ns corresponds to a hydrodynamic radius of 4.4 nm and results in a
+      relaxation time :math:`\Gamma= 1/(q^2D) \approx 58 \mu s`.
+    - The below plots shows the CONTIN result as correlation time distribution
+      with mean 58 µs ±12 µs and peak weight 95% in good approximation.
+    - The second plot shows the radius distribution for different weights.
+      Probabilities are for an interval around a point x value that the sum of all points is normalized.
+      (not the integral on the log scale).
+    - For real measurements one may cut noisy short times or dust contaminated  long times.
 
     ::
 
      import jscatter as js
      import numpy as np
      t=js.loglist(1,10000,1000)   #times in microseconds
-     q=4*np.pi/1.333/632*np.sin(np.pi/2) # 90 degrees for 632 nm , unit is 1/nm**2
+     q=4*np.pi*1.333/632*np.sin(np.deg2rad(90)/2) # 90 degrees for 632 nm , unit is 1/nm**2
      D=0.05*1000  # nm**2/ns * 1000 = units nm**2/microseconds
      noise=0.0001  # typical < 1e-3
      data=js.dA(np.c_[t,0.95*np.exp(-q**2*D*t)+noise*np.random.randn(len(t))].T)
@@ -357,29 +353,34 @@ def contin(datalist, Ngrid=256, tmin=-2, tmax=-2, bgr=0, distribution='dls', RDG
      js.dls.contin_display(dr)         # display overview
 
      # same to demonstrate access to distributions
-     # plot first result in dr list
      bf =dr[0].contin_bestFit
-     p=js.grace(1.,1.3)
-     p.multi(2,1)
+     p=js.grace(1.,1.)
+     p.multi(2,1,vgap=0.25)
+
+     # access correlation function and relaxation time distribution
      p[0].plot(data,sy=[1,1,1],legend='simulated data')
-     p[0].xaxis(scale='log',label=r'\xG\f{} / µs',min=1,max=1e5)
-     p[0].yaxis(label=r'G\s1\N / P(\xG\f{})')
      p[0].plot(dr[0],li=[1,3,3],sy=0,le='contin result')
      p[0].plot(bf.X,bf.Y*10,sy=[2,0.5,2],le='relaxation time distribution')
+     p[0].xaxis(scale='log',label=r'\xG\f{} / µs',min=1,max=1e5)
+     p[0].yaxis(label=r'G\s1\N / P(\xG\f{})',min=0,max=1.1)
      p[0].legend(x=300,y=0.7)
+
+     # Hydrodynamic radius distribution in different weights
      p[1].plot(bf[3],bf[1],sy=[2,0.5,2],li=[1,1,''],le='intensity weight')
      p[1].plot(bf[3],bf[4],sy=[2,0.5,3],li=[1,1,''],le='mass weight')
      p[1].plot(bf[3],bf[5],sy=[2,0.5,4],li=[1,1,''],le='number weight')
      p[1].xaxis(scale='log',label=r'R\sh\N / cm',min=1e-7,max=1e-5)
-     p[1].yaxis(label=r'P(R\sh\N)')
-     p[1].legend(x=1.5e-6,y=0.08)
-     p[1].text(f'Rh from contin {bf.ipeaks[0,6]:.2g}',x=1.3e-6,y=0.04)
+     p[1].yaxis(label=r'P(R\sh\N)',min=0,max=0.1)
+     p[1].legend(x=1.5e-6,y=0.1)
+
+     # Rh of first peak
+     p[1].text(f'Rh from contin {bf.ipeaks[0,6]:.2g} cm',x=1.3e-6,y=0.04)
      # p.save(js.examples.imagepath+'/contin.jpg')
 
      # access peak values by
      dr[0].contin_bestFit.ipeaks_name
      dr[0].contin_bestFit.ipeaks
-     # calc std/mean of peak
+     # calc std/mean of first peak
      dr[0].contin_bestFit.ipeaks[0,2]/dr[0].contin_bestFit.ipeaks[0,1]
 
     .. image:: ../../examples/images/contin.jpg
@@ -398,7 +399,7 @@ def contin(datalist, Ngrid=256, tmin=-2, tmax=-2, bgr=0, distribution='dls', RDG
      import jscatter as js
      import numpy as np
      t=js.loglist(0.125,10000,1000)   #times in microseconds
-     q=4*np.pi/1.333/632*np.sin(np.pi/2) # 90 degrees for 632 nm , unit is 1/nm**2
+     q=4*np.pi*1.333/632*np.sin(np.deg2rad(90)/2) # 90 degrees for 632 nm , unit is 1/nm**2
      D=0.05*1000  # nm**2/ns * 1000 = units nm**2/microseconds
      noise=0.0001  # typical < 1e-3
      data=js.dA(np.c_[t,0.45*np.exp(-q**2*D*t)+0.45*np.exp(-q**2*D*20*t) +noise*np.random.randn(len(t))].T)
@@ -413,30 +414,35 @@ def contin(datalist, Ngrid=256, tmin=-2, tmax=-2, bgr=0, distribution='dls', RDG
      dr=js.dls.contin(data,distribution='x')
 
      # same to demonstrate access to distributions
-     # plot first result in dr list
      bf =dr[0].contin_bestFit
-     p=js.grace(1.,1.3)
-     p.multi(2,1)
+     p=js.grace(1.,1.)
+     p.multi(2,1,vgap=0.25)
+
+     # access correlation function and relaxation time distribution
      p[0].plot(data,sy=[1,1,1],legend='simulated data')
      p[0].xaxis(scale='log',label=r'\xG\f{} / µs',min=1,max=1e5)
      p[0].yaxis(label=r'G\s1\N / P(\xG\f{})')
      p[0].plot(dr[0],li=[1,3,3],sy=0,le='contin result')
      p[0].plot(bf.X,bf.Y*10,sy=[2,0.5,2],le='relaxation time distribution')
-     p[0].legend(x=100,y=0.7)
-     p[1].plot(bf[3],bf[1],sy=[2,0.5,2],li=[1,1,''],le='intensity weight')
-     p[1].plot(bf[3],bf[4],sy=[2,0.5,3],li=[1,1,''],le='mass weight')
-     p[1].plot(bf[3],bf[5],sy=[2,0.5,4],li=[1,1,''],le='number weight')
+     p[0].legend(x=20,y=0.8)
+
+     # Hydrodynamic radius distribution in different weights
+     p[1].plot(bf._hydrodynamicradii,bf._intensityweightci, sy=[2,0.5,2],li=[1,1,''],le='intensity weight')
+     p[1].plot(bf._hydrodynamicradii,bf._massweightci, sy=[2,0.5,3],li=[1,1,''],le='mass weight')
+     p[1].plot(bf._hydrodynamicradii,bf._numberweightci, sy=[2,0.5,4],li=[1,1,''],le='number weight')
      p[1].xaxis(scale='log',label=r'R\sh\N / cm',min=1e-9,max=1e-5)
-     p[1].yaxis(label=r'P(R\sh\N)')
-     p[1].legend(x=1.5e-7,y=0.08)
-     p[1].text(f'Rh from contin {bf.ipeaks[0,6]:.3g}',x=1.3e-7,y=0.04)
+     p[1].yaxis(label=r'P(R\sh\N)',min=0,max=0.12)
+     p[1].legend(x=1.5e-7,y=0.1)
+
+     # Rh of first two peaks
+     p[1].text(f'Rh_1 ={bf.ipeaks[0,6]:.3g} cm \\nRh_2 ={bf.ipeaks[1,6]:.3g} cm',x=1.3e-7,y=0.04)
      # p.save(js.examples.imagepath+'/contin_bimodal.jpg')
 
      # Peak integration resulting in about 0.5 for both
      p1=(1<bf.X) &(bf.X<20) # peak borders 10 and 100
-     fp1=np.trapz(bf.Y[p1],bf.X[p1])/np.trapz(ff.Y,ff.X) #
-     p2=(40<ff.X) &(ff.X<400) # peak borders 10 and 100
-     fp2=np.trapz(ff.Y[p2],ff.X[p2])/np.trapz(ff.Y,ff.X) #
+     fp1=np.trapz(bf.Y[p1],bf.X[p1])/np.trapz(bf.Y,bf.X) #
+     p2=(40<bf.X) &(bf.X<400) # peak borders 10 and 100
+     fp2=np.trapz(bf.Y[p2],bf.X[p2])/np.trapz(bf.Y,bf.X) #
 
 
     .. image:: ../../examples/images/contin_bimodal.jpg
@@ -541,11 +547,9 @@ def contin(datalist, Ngrid=256, tmin=-2, tmax=-2, bgr=0, distribution='dls', RDG
 
     """
     # contin is called in a shell like:  contin <input.txt >output.txt
-    # we mimic this by subprocess.Popen
-    # create a pipe to CONTIN with the input to stdin and stdout to
-    # outputpath = os.path.realpath(os.path.dirname(__file__))
+    # we mimic this by subprocess.run
 
-    if continexe == '':
+    if continexe is None:
         raise Exception('There is no contin executable found. ' +
                         'Please compile and place executable at callable path. ' +
                         'See documentation of DLS module.')
@@ -580,7 +584,7 @@ def contin(datalist, Ngrid=256, tmin=-2, tmax=-2, bgr=0, distribution='dls', RDG
         if T < 273:
             print('Warning: temperature below ZERO!! ')
     elif 'R18' in kw:
-        T = kw['R18']  # here we mis use the T as the proportionality constant for kernel molweight
+        T = kw['R18']  # here we misuse the T as the proportionality constant for kernel molweight
     else:
         T = 293.  # in K
     if 'n' in kw:
@@ -637,7 +641,8 @@ def contin(datalist, Ngrid=256, tmin=-2, tmax=-2, bgr=0, distribution='dls', RDG
         R21 = kb * T / (0.06 * pi * _visc(T, **kw)) * qq(n, l, a) ** 2  # D*Rh * Q**2
         R22 = -1
         R23 = 0
-    # write header für CONTIN als ASCII inputfile           #######################################
+
+    # write a common header for CONTIN as ASCII inputfile
     last = 1  # for multi dataset evaluation this is -1 (false) except for the last one
     # we process here ONLY single files
     elements = 40  # just to have an array for it; last 2 lines are "end" and NY
@@ -729,7 +734,7 @@ def contin(datalist, Ngrid=256, tmin=-2, tmax=-2, bgr=0, distribution='dls', RDG
         transk = lambda t, n, l, a, T, v, R22, R21: (t * R21) ** (-1 / R22)
 
     ###################################
-    # now look at the data
+    # now look at the data and modify also the header if necessary
     idata = 0
     print('processing  %i datasets ' % len(datalist))
     for data in datalist:
@@ -817,8 +822,9 @@ def contin(datalist, Ngrid=256, tmin=-2, tmax=-2, bgr=0, distribution='dls', RDG
         header[3] = 'GMNMX'.ljust(6) + str(2).rjust(5) + ('%15.4E' % max(gmin, gmax))  # fit interval max
         if 'T' not in kw and 'R18' not in kw:
             try:
-                T = data.Temperature[0]
+                T = _getfirst(data.Temperature)
             except:
+                print('No temperature given, set to default 293.15K')
                 T = 273.15 + 20
             header[19] = 'RUSER'.ljust(6) + str(18).rjust(5) + ('%15.4E' % T)
         header[16] = 'RUSER'.ljust(6) + str(15).rjust(5) + ('%15.4E' % n)
@@ -826,12 +832,6 @@ def contin(datalist, Ngrid=256, tmin=-2, tmax=-2, bgr=0, distribution='dls', RDG
         header[18] = 'RUSER'.ljust(6) + str(17).rjust(5) + ('%15.4E' % a)
         header[20] = 'RUSER'.ljust(6) + str(19).rjust(5) + ('%15.4E' % v)
 
-        p = subprocess.Popen(continexe, shell=True,
-                             bufsize=0,
-                             stdin=subprocess.PIPE,
-                             stdout=subprocess.PIPE,
-                             stderr=subprocess.PIPE)
-
         # set the number of values
         lenX = len(data.X[itmin:itmax])
         header[-1] = 'NY'.ljust(6) + str(lenX).rjust(5)
@@ -850,22 +850,24 @@ def contin(datalist, Ngrid=256, tmin=-2, tmax=-2, bgr=0, distribution='dls', RDG
                 f.writelines(input.getvalue().decode('utf-8'))
 
         # now run contin in a shell like environment with pipes
-        # with input in stdin and collect the stout and error in output and error
-        (output, error) = p.communicate(input.getvalue())
-        output = output.decode('utf-8')
-        error = error.decode('utf-8')
+        p = subprocess.run(continexe, input=input.getvalue(), capture_output=True)
         input.close()
-        if error != '':
-            print(file, '      :', error)
-        if len(output) == 0:
-            print('there was nothing in output yet')
-            return
+        output = p.stdout.decode('utf-8')
+        error = p.stderr.decode('utf-8')
+        if p.returncode >0 or len(output) == 0:
+            if error != '':
+                for line in error.split('\n'):
+                    print('contin_std_err>', line)
+            if len(output) == 0:
+                print('there was nothing in output yet')
+                return
+
         # to check output
         if 'write' in kw or 'w' in kw:
             with open('./' + file + '.con', 'w') as f:
                 f.writelines(output)
 
-        # CONTIN finished and we look at the output ------------------------------------
+        # CONTIN finished, we look at the output ------------------------------------
 
         # sort the output to the data
         outblocks = output.split('UNREGULARIZED VARIABLES')[-1].split(file)
@@ -971,7 +973,7 @@ def contin(datalist, Ngrid=256, tmin=-2, tmax=-2, bgr=0, distribution='dls', RDG
             data.contin_alphalist.index(data.contin_alpha)]  # this is the choosen solution
         data.contin_bestFit.imaximum = data.contin_bestFit[0].searchsorted(data.contin_bestFit[1].max())
         data.contin_bestFit.columnname = \
-            'relaxationtimes;intensityweightci; errors; hydrodynamicradii; massweightci;numberweightci'
+            'relaxationtimes; intensityweightci; errors; hydrodynamicradii; massweightci; numberweightci'
         # a short info about the peaks
         ipeaks = np.c_[[[p['weight'],
                          p['mean'],
@@ -1198,7 +1200,7 @@ def readZetasizerNano(filename, NANOcoding="cp1257", NANOlocale=None):
        - .distributions : Distribution fits with columns
           [RelaxationTimes, Sizes, Intensities, Volumes, Diffusions] (if present)
        - fit results of correlation function in attributes (some channels are discarded -> see Zetasizer settings)
-           - .distributionFitDelayTimes timechanels
+           - .distributionFitDelayTimes timechannels
            - .distributionFitData measured points
            - .distributionFit fit result
        - Check .attr for more attributes like
@@ -1245,7 +1247,7 @@ def readZetasizerNano(filename, NANOcoding="cp1257", NANOlocale=None):
      p[1].legend(x=50,y=30)
      p[0].title('Zetasizer data with distribution fit')
      p[0].subtitle('Results from instrument software')
-     # p.save(js.examples.imagepath+'/Zetasizerexample.jpg',format='jpeg',size=[600,600]) #as jpg file
+     # p.save(js.examples.imagepath+'/Zetasizerexample.jpg',format='jpeg',size=[600/300,600/300]) #as jpg file
 
     .. image:: ../../examples/images/Zetasizerexample.jpg
          :width: 50 %
