@@ -2,7 +2,7 @@
 # written by Ralf Biehl at the Forschungszentrum Jülich ,
 # Jülich Center for Neutron Science 1 and Institute of Complex Systems 1
 #    Jscatter is a program to read, analyse and plot data
-#    Copyright (C) 2015-2019  Ralf Biehl
+#    Copyright (C) 2015-2021  Ralf Biehl
 #
 #    This program is free software: you can redistribute it and/or modify
 #    it under the terms of the GNU General Public License as published by
@@ -18,45 +18,21 @@
 #    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
 """
-Functions for parallel computations on a single multi core machine using the standard library multiprocessing.
-
-Not the programming details, but the way how to speed up some things.
-
-- If your computation is already fast (e.g. <1s) go on without parallelisation.
-  In an optimal case you gain a speedup as the number of cpu cores.
-- If you want to use a cluster with all cpus, this is not the way (you need MPI).
-
-Parallelisation is no magic and this module is for convenience for non specialist of parallel computing.
-The main thing is to pass additional parameters to the processes (a pool of workers)
-and loop only over one parameter given as list.
-Opening and closing of the pool is hidden in the function.
-In this way we can use a multicore machine with all cpus.
-
-During testing I found that shared memory does not really speed up,
-if we just want to calculate a function e.g. for a list of different
-Q values dependent on model parameters. Here the pickling of numpy arrays is
-efficient enough compared to the computation we do.
-The amount of data pickled should not be too large as each process
-gets a copy and pickling needs time.
-
-If speed is an issue and shared memory gets important i advice using Fortran with OpenMP
-as used for ff.cloudScattering with parallel computation and shared memory.
-For me this was easier than the different solutions around.
-
-We use here only non modified input data and return a new dataset,
-so we dont need to care about what happens if one process changes the data needed
-in another process (race conditions,...), anyway its not shared.
-Please keep this in mind and dont complain if you find a way to modify input data.
-
-For easier debugging (to find the position of an error in the pdb debugger) use the option debug.
-In this case the multiprocessing is not used and the debugger finds the error correctly.
-
-See example in doForList.
+Functions for parallel computations on a single multicore machine using the standard library multiprocessing
+and some related stuff.
 
 """
 
+from math import log, floor, ceil, fmod
 import scipy.constants as constants
 import multiprocessing as mp
+
+try:
+    from multiprocessing import shared_memory
+except ImportError:
+    # compatibility for Python3.7
+    shared_memory = None
+
 import numpy as np
 
 try:
@@ -68,7 +44,9 @@ except ImportError:
 
 _Nrand = np.random.random_integers
 
-from math import log, floor, ceil, fmod
+__all__ = ['haltonSequence', 'randomPointsOnSphere', 'randomPointsInCube',
+           'fibonacciLatticePointsOnSphere', 'sphereAverage', 'doForList',
+           'shared_create', 'shared_recover', 'shared_close']
 
 
 def haltonSequence(size, dim, skip=0):
@@ -112,7 +90,7 @@ def haltonSequence(size, dim, skip=0):
      ax = fig.add_subplot(111, projection='3d')
      for i,color in enumerate(['b','g','r','y']):
         # create halton sequence and shift it to needed shape
-        pxyz=js.parallel.haltonSequence(400,3).T*2-1
+        pxyz=js.formel.haltonSequence(400,3).T*2-1
         ax.scatter(pxyz[:,0],pxyz[:,1],pxyz[:,2],color=color,s=20)
      ax.set_xlim([-1,1])
      ax.set_ylim([-1,1])
@@ -195,7 +173,7 @@ def randomPointsOnSphere(NN, r=1, skip=0):
      fig = plt.figure()
      ax = fig.add_subplot(111, projection='3d')
      for i,color in enumerate(['b','g','r','y']):
-        points=js.parallel.randomPointsOnSphere(400,skip=400*i)
+        points=js.formel.randomPointsOnSphere(400,skip=400*i)
         points=points[points[:,1]>0,:]
         pxyz=js.formel.rphitheta2xyz(points)
         ax.scatter(pxyz[:,0],pxyz[:,1],pxyz[:,2],color=color,s=20)
@@ -283,9 +261,9 @@ def randomPointsInCube(NN, skip=0, dim=3):
      fig = plt.figure()
      ax = fig.add_subplot(111, projection='3d')
      N=30
-     cubes=js.parallel.randomPointsInCube(20)*3
+     cubes=js.formel.randomPointsInCube(20)*3
      for i,color in enumerate(['b','g','r','y','k']*3):
-        points=js.parallel.randomPointsInCube(N,skip=N*i).T
+        points=js.formel.randomPointsInCube(N,skip=N*i).T
         pxyz=points*0.3+cubes[i][:,None]
         ax.scatter(pxyz[0,:],pxyz[1,:],pxyz[2,:],color=color,s=20)
      ax.set_xlim([0,3])
@@ -367,6 +345,7 @@ def fibonacciLatticePointsOnSphere(NN, r=1):
      plt.show(block=False)
 
     """
+    NN = int(NN)
     return np.c_[(r * np.ones(2 * NN + 1),
                   np.mod((2 * np.pi * np.r_[-NN:NN + 1] / constants.golden) + np.pi, 2 * np.pi) - np.pi,
                   np.arcsin(2 * np.r_[-NN:NN + 1] / (2 * NN + 1.)) + np.pi / 2)]
@@ -398,9 +377,9 @@ def rphitheta2xyz(RPT):
 
 
 # noinspection PyIncorrectDocstring
-def psphereAverage(funktion, relError=300, *args, **kwargs):
+def sphereAverage(funktion, relError=300, *args, **kwargs):
     """
-    Parallel evaluation of spherical average of function.
+    Vectorized spherical average of funktion, parallel or non-parallel.
     
     A Fibonacci lattice or Monte Carlo integration with pseudo random grid is used.
     
@@ -408,24 +387,50 @@ def psphereAverage(funktion, relError=300, *args, **kwargs):
     ----------
     funktion : function
         Function to evaluate.
-        Function first argument gets cartesian coordinate [x,y,z] of point on unit sphere.
+        Funktion first argument gets cartesian coordinate [x,y,z] of point on unit sphere.
     relError : float, default 300
         Determines how points on sphere are selected
          - >1  Fibonacci Lattice with relError*2+1 points
          - 0<1 Pseudo random points on sphere (see randomPointsOnSphere).
-               Stops if relative improvement in mean is less than relError (uses steps of 40 new points).
+               Stops if relative improvement in mean is less than relError
+               (uses steps of 40 or (20ncpu) new points).
                Final error is (stddev of N points) /sqrt(N) as for Monte Carlo methods
                even if it is not a correct 1-sigma error in this case.
-    arg,kwargs : 
+
+               This has to be used with care as convergence might be slow if *std* is
+               intrinsically large in calculated values.
+    ncpu : int, optional
+        Number of cpus in the pool if started as main process (otherwise single process).
+         - not given or 0   -> default, all cpus are used
+         - int>0      min (ncpu, mp.cpu_count)
+         - int<0      ncpu not to use
+        Please check if multiprocessing is really useful.
+        For simple calculations it's faster not to use it (avoiding pickle overhead).
+        The below first example is around 10times faster on a single core (ncpu=1).
+        Think about passPoints.
+
+    passPoints : bool
+        If the function accepts a Nx3 array of points these will be passed instead of single points.
+        The function return value should index the points result in axis 0.
+        This might speed up dependent on the function (e.g. formfactor prism) .
+
+        Sets ncpu=1 .
+    arg,kwargs :
         forwarded to function
     
     Returns
     -------
-    array like with values from function and appended error
+    array
+        Values from function and appended Monte Carlo error estimates.
+         - If an array of values is returned in function, the result is also an array with double size.
+
+           To separate use `values, error = res.reshape(2,-1)`
+         - If list of array is returned a list is returned with values and error.
+
 
     Notes
     -----
-     - Works also on single core machines.
+     - Works also on single core machines and with single process (avoiding creation of a pool of workers).
      - For integration over a continuous function as a form factor in scattering the random
        points are not statistically independent. Think of neighbouring points on an isosurface
        which are correlated and therefore the standard deviation is biased. In this case the
@@ -434,64 +439,138 @@ def psphereAverage(funktion, relError=300, *args, **kwargs):
 
     Examples
     --------
+    Parallel sphere average
+
+    Conversion to rpt coordinates and sum results in average value of these coordinate in result.
+
+    Error of radius is machine precision, phi,theta errors are typical Monte Carlo error
     ::
 
+     import jscatter as js
      def f(x,r):
-        return [js.formel.xyz2rphitheta(x)[1:].sum()*r]
-     js.parallel.psphereAverage(f,relError=500,r=1)
-     js.parallel.psphereAverage(f,relError=0.01,r=1)
+        # return a list of values to integrate
+        return [js.formel.xyz2rphitheta(x)*r]
+     val, error = js.formel.sphereAverage(f,relError=500,r=1)
+     # => [1, 0, pi/2 ]
+     js.formel.sphereAverage(f,relError=0.01,r=1)
 
+    ::
+
+     import jscatter as js
+     import numpy as np
+
+     def fp(singlepoint):
+        # return a list of values to integrate
+        return js.formel.xyz2rphitheta(singlepoint)[1:]
+     val,error = js.formel.sphereAverage(fp,relError=500).reshape(2,-1)
+     val,error = js.formel.sphereAverage(fp,relError=0.1).reshape(2,-1)
+
+    Pass all points to one process, not parallel.
+
+    ::
+     def fps(allpoints,r):
+        res = js.formel.xyz2rphitheta(allpoints)
+        return res
+    val,error =  js.formel.sphereAverage(fps,r=1,passPoints=1).reshape(2,-1)
 
     """
+    passPoints = kwargs.pop('passPoints', False)
+
     if relError < 0:
         relError = abs(relError)
-    if 'ncpu' in kwargs:
-        ncpu = kwargs['ncpu']
-        del kwargs['ncpu']
-    else:
-        ncpu = mp.cpu_count()
-    pool = mp.Pool(ncpu)
-    steps = ncpu * 20
     if relError == 0:
         print('Try again with relError > 0')
         return
-    elif 0 < abs(relError) < 1:
-        points = rphitheta2xyz(randomPointsOnSphere(NN=steps))
-        npoints = steps
-        jobs = [pool.apply_async(funktion, (point,) + args, kwargs) for point in points]
-        result = [np.asarray(job.get(0xFFFF)) for job in jobs]
-        prevmean = np.r_[result].mean(axis=0).real
-        # prevstd =result.std(axis=0).real
-        while 1:
-            points = rphitheta2xyz(randomPointsOnSphere(NN=steps, skip=npoints))
-            npoints += steps
-            jobs = [pool.apply_async(funktion, (point,) + args, kwargs) for point in points]
-            result += [np.asarray(job.get(0xFFFF)) for job in jobs]
-            results = np.array(result)
-            mean = results.mean(axis=0).real
-            # std=results.std(axis=0).real
-            if np.all(abs(1 - abs(prevmean / mean)) < relError):
-                # abs(results[:,0].std()/results[:,0].mean()/np.sqrt(results.shape[0]-1))<relError
-                # and results.shape[0]>42:
-                break
-            prevmean = mean
-            # prevstd  = std
-    elif relError > 1:
-        qfib = fibonacciLatticePointsOnSphere(relError, 1)
-        points = rphitheta2xyz(qfib)  # to cartesian
-        jobs = [pool.apply_async(funktion, (point,) + args, kwargs) for point in points]
-        results = np.r_[[np.asarray(job.get(0xFFFF)) for job in jobs]]
-    # clean up
-    pool.close()
-    pool.join()
-    return np.r_[results.mean(axis=0), results.std(axis=0) / np.sqrt(np.shape(results[:, 0])[0])]
+
+    ncpu = mp.cpu_count()
+    # except force to something else
+    if 'ncpu' in kwargs:
+        if kwargs['ncpu'] < 0:
+            ncpu = max(ncpu + kwargs['ncpu'], 1)
+        elif kwargs['ncpu'] > 0:
+            ncpu = min(ncpu, kwargs['ncpu'])
+        del kwargs['ncpu']
+
+    if 'debug' in kwargs:
+        _ = kwargs.pop('debug', None)
+        ncpu=1
+
+    if ncpu == 1 or mp.current_process().name != 'MainProcess' or passPoints:
+        # non parallel
+        if 0 < relError < 1:
+            steps = 40
+            points = rphitheta2xyz(randomPointsOnSphere(NN=steps, skip=0))
+            npoints = steps
+            if passPoints:
+                results = np.array(funktion(points, *args, **kwargs), ndmin=1)
+            else:
+                results = np.r_[[np.array(funktion(point, *args, **kwargs), ndmin=1) for point in points]]
+            prevmean = results.mean(axis=0).real
+
+            while 1:
+                points = rphitheta2xyz(randomPointsOnSphere(NN=steps, skip=npoints))
+                npoints += steps
+                if passPoints:
+                    result = np.array(funktion(points, *args, **kwargs), ndmin=1)
+                else:
+                    result = np.r_[[np.array(funktion(point, *args, **kwargs), ndmin=1) for point in points]]
+                results = np.r_[results, result]
+                mean = results.mean(axis=0).real
+                # var = results.var(axis=0,ddof=1)**0.5
+                if np.all(abs(mean - prevmean) < relError * abs(mean)):
+                    break
+                prevmean = mean
+
+        elif relError >= 1:
+            qfib = fibonacciLatticePointsOnSphere(max(relError, 7), 1)
+            points = rphitheta2xyz(qfib)  # to cartesian
+            if passPoints:
+                results = np.array(funktion(points, *args, **kwargs), ndmin=2)
+            else:
+                results = np.r_[[np.array(funktion(point, *args, **kwargs), ndmin=1) for point in points]]
+
+        return np.r_[results.mean(axis=0), results.std(axis=0) / np.sqrt(results.shape[0] - 1)]
+
+    else:
+        with mp.Pool(processes=ncpu) as pool:
+            steps = ncpu * 20
+            if 0 < abs(relError) < 1:
+                points = rphitheta2xyz(randomPointsOnSphere(NN=steps))
+                npoints = steps
+                jobs = [pool.apply_async(funktion, (point,) + args, kwargs) for point in points]
+                result = [np.asarray(job.get(0xFFFF)) for job in jobs]
+                prevmean = np.r_[result].mean(axis=0).real
+                # prevstd =result.std(axis=0).real
+                while 1:
+                    points = rphitheta2xyz(randomPointsOnSphere(NN=steps, skip=npoints))
+                    npoints += steps
+                    jobs = [pool.apply_async(funktion, (point,) + args, kwargs) for point in points]
+                    result += [np.asarray(job.get(0xFFFF)) for job in jobs]
+                    results = np.array(result)
+                    mean = results.mean(axis=0).real
+                    # std=results.std(axis=0).real
+                    if np.all(abs(1 - abs(prevmean / mean)) < relError):
+                        # abs(results[:,0].std()/results[:,0].mean()/np.sqrt(results.shape[0]-1))<relError
+                        # and results.shape[0]>42:
+                        break
+                    prevmean = mean
+                    # prevstd  = std
+
+            elif relError > 1:
+                qfib = fibonacciLatticePointsOnSphere(relError, 1)
+                points = rphitheta2xyz(qfib)  # to cartesian
+                jobs = [pool.apply_async(funktion, (point,) + args, kwargs) for point in points]
+                results = np.r_[[np.asarray(job.get(0xFFFF)) for job in jobs]]
+
+        return np.r_[results.mean(axis=0), results.std(axis=0) / np.sqrt(np.shape(results[:, 0])[0])]
 
 
 # noinspection PyIncorrectDocstring
 def doForList(funktion, looplist, *args, **kwargs):
     """
+    Parallel excecution of a function in a pool of workers to speed up (multiprocessing).
+
     Apply function with values in looplist in a pool of workers in parallel using multiprocessing.
-    
     Like multiprocessing map_async but distributes automatically all given arguments.
     
     Parameters
@@ -501,12 +580,16 @@ def doForList(funktion, looplist, *args, **kwargs):
         Return value of function should contain parameters or at least the loopover value to allow a check, if desired.
     loopover : list of string, default= None
         Names of arguments to use for (sync) looping over with values in looplist.
-        - If not given the first funktion argument is used.
-        - If loopover is single argument this gets looplist[i,:] .
-    looplist : list or array N x len(loopover)
+         - If not given the first funktion argument is used.
+         - If loopover is single argument this gets looplist[i,:]
+         - Two arguments to loopover : ``loopover=['q', 'iq']``
+    looplist : list or list of tuples with len(loopover)
         List of values to loop over.
+         - single argument just use a list ``[0.01,0.1,0.2,0.3,0.4]``
+         - for above 'q', 'iq' maybee ``lopplist=[(q, i) for i, q in enumerate([0.01,0.1,0.2,0.3,0.4])]``
+
     ncpu : int, optional
-        Number of cpus in the pool.
+        Number of cpus in the pool if started as main process (otherwise single process).
          - not given or 0   -> all cpus are used
          - int>0      min (ncpu, mp.cpu_count)
          - int<0      ncpu not to use
@@ -520,16 +603,58 @@ def doForList(funktion, looplist, *args, **kwargs):
     Returns
     -------
     list : list of function return values as  [result1,result2,.....]
+
            The order of return values is not explicitly synced to looplist.
+           The return array may be prepended with the value looplist[i] as reference.
+           E.g.::
+
+             def f(x,a,b,c,d):
+             result = x+a+b+c+d
+                 return [x, result]
 
     Notes
     -----
-    The return array of function may be prepended with the value looplist[i] as reference.
-    E.g.::
+    Functions for parallel computations on a single multicore machine using the standard library multiprocessing.
+
+    Not the programming details, but the way how to speed up some things.
+
+    - If your computation is already fast (e.g. <1s) go on without parallelisation.
+      In an optimal case you gain a speedup as the number of cpu cores.
+    - If you want to use a cluster with all cpus, this is not the way (you need MPI).
+
+    Parallelisation is no magic and this module is for convenience for non-specialist of parallel computing.
+    The main thing is to pass additional parameters to the processes (a pool of workers)
+    and loop only over one parameter given as list.
+    Opening and closing of the pool is hidden in the function.
+    In this way we can use a multicore machine with all cpus.
+
+    During testing, I found that shared memory does not really speed up in most cases,
+    if we just want to calculate a function e.g. for a list of different
+    Q values dependent on model parameters. Here the pickling of numpy arrays is
+    efficient enough compared to the computation we do.
+    The amount of data pickled should not be too large as each process
+    gets a copy and pickling needs time.
+
+    If speed is an issue and shared memory gets important i advice using Fortran with OpenMP
+    as used for ff.cloudScattering with parallel computation and shared memory.
+    For me this was easier than the different solutions around.
+
+    We use here only non-modified input data and return a new dataset,
+    so we don't need to care about what happens if one process changes the data needed
+    in another process (race conditions,...), anyway it's not shared.
+    Please keep this in mind and don't complain if you find a way to modify input data.
+
+    For easier debugging (to find the position of an error in the pdb debugger) use the option debug.
+    In this case the multiprocessing is not used and the debugger finds the error correctly.
+
+
+    **Shared memory**
+
+    Shared memory can be created and used with :py:func:`~jscatter.parallel.shared_create` ,
+    :py:func:`~jscatter.parallel.shared_recover` and :py:func:`~jscatter.parallel.shared_close`.
+
+    See notes and example in :py:func:`~jscatter.parallel.shared_create`.
 
-     def f(x,a,b,c,d):
-     result = x+a+b+c+d
-         return [x, result]
 
     Examples
     --------
@@ -543,23 +668,25 @@ def doForList(funktion, looplist, *args, **kwargs):
         return [x,res]
 
      # loop over first argument, here x
-     res = js.parallel.doForList(f,looplist=np.arange(100),a=1,b=2,c=3,d=11)
+     res = js.formel.doForList(f,looplist=np.arange(100),a=1,b=2,c=3,d=11)
      # loop over 'd' ignoring the given d=11 (which can be omitted here)
-     res = js.parallel.doForList(f,looplist=np.arange(100),loopover='d',x=0,a=1,b=2,c=3,d=11)
+     res = js.formel.doForList(f,looplist=np.arange(100),loopover='d',x=0,a=1,b=2,c=3,d=11)
 
      # using a list of 2 values for x (is first argument)
      def f(x,a,b,c,d):
         res=x[0]+x[1]+a+b+c+d
         return [x[0],res]
      loop = np.arange(100).reshape(-1,2)  # has 2 values fin second dimension
-     res = js.parallel.doForList(f,looplist=loop,a=1,b=2,c=3,d=11)
+     res = js.formel.doForList(f,looplist=loop,a=1,b=2,c=3,d=11)
 
      # looping over several variables in sync
      loop = np.arange(100).reshape(-1,2)
-     res = js.parallel.doForList(f,looplist=loop,loopover=['a','b'],x=[100,200],a=1,b=2,c=3,d=11)
+     res = js.formel.doForList(f,looplist=loop,loopover=['a','b'],x=[100,200],a=1,b=2,c=3,d=11)
 
     """
     output = kwargs.pop('output', True)
+    looplen = len(looplist)
+
     ncpu = mp.cpu_count()
     if 'ncpu' in kwargs:
         if kwargs['ncpu'] < 0:
@@ -568,17 +695,21 @@ def doForList(funktion, looplist, *args, **kwargs):
             ncpu = min(ncpu, kwargs['ncpu'])
         del kwargs['ncpu']
 
+    # use not more cpu than elemtents in looplist
+    ncpu = min(ncpu, looplen)
+
     # check for callback
     cb = kwargs.pop('cb', None)
 
     # not given defaults to first varname in funktion
     loopover = kwargs.pop('loopover', funktion.__code__.co_varnames[0])
 
-    if ('debug' in kwargs and kwargs['debug'] > 0) or ncpu == 1:  # in sequence for testing purposes
+    if ('debug' in kwargs and kwargs['debug'] > 0) or ncpu == 1 or mp.current_process().name != 'MainProcess':
+        # in sequence for testing purposes
         _ = kwargs.pop('debug', None)
         res = []
         if output:
-            print('start NO pool; this is a test ')
+            print('start NO pool')
         if isinstance(loopover, str):
             for Q in looplist:
                 res.append(funktion(*args, **dict(kwargs, **{loopover: Q})))
@@ -597,93 +728,133 @@ def doForList(funktion, looplist, *args, **kwargs):
         if output:
             print('start pool of ', ncpu)
 
-        pool = mp.Pool(ncpu)
-        # we need to create a temporary dict that is given to a job and is not only a view to the updated kwargs
-        # this is needed as the view changes to each new value in the loop even in the jobs
-        if isinstance(loopover, str):
-            jobs = [pool.apply_async(funktion, args, dict(kwargs, **{loopover: Q}), callback=cb) for Q in looplist]
-        elif isinstance(loopover, (list, tuple)) and (len(loopover) == 1):
-            jobs =[pool.apply_async(funktion, args, dict(kwargs, ** {loopover[0]: Q}), callback=cb) for Q in looplist]
-        else:
-            jobs = [pool.apply_async(funktion, args, dict(kwargs, **dict(zip(loopover, Q))), callback=cb)
-                    for Q in looplist]
-        res = [job.get() for job in jobs]
+        with mp.Pool(ncpu) as pool:
+            # we need to create a temporary dict that is given to a job and is not only a view to the updated kwargs
+            # this is needed as the view changes to each new value in the loop even in the jobs
+            if isinstance(loopover, str):
+                # a single parameter name
+                jobs = [pool.apply_async(funktion, args, dict(kwargs, **{loopover: Q}), callback=cb)
+                        for Q in looplist]
+            elif isinstance(loopover, (list, tuple)) and (len(loopover) == 1):
+                # list with one paramter name
+                jobs = [pool.apply_async(funktion, args, dict(kwargs, **{loopover[0]: Q}), callback=cb)
+                        for Q in looplist]
+            else:
+                # several parameter names to sync loop
+                jobs = [pool.apply_async(funktion, args, dict(kwargs, **dict(zip(loopover, Q))), callback=cb)
+                        for Q in looplist]
+            # get results
+            res = [job.get() for job in jobs]
 
-        # clean up
-        pool.close()
-        pool.join()
         if output:
             print('closed pool again')
     return res
 
 
-# noinspection PyIncorrectDocstring
-def doForQlist(funktion, qList, *args, **kwargs):
+def shared_create(ar):
     """
-    Calculates for qlist the function in a pool of workers using multiprocessing.
-    
-    Calcs [function(Qi, *args, **kwargs) for Qi in qlist ] in parallel.
-    The return value of function will contain the value Qi as reference.
+    Create shared memory array before sending into pool of workers or usage in doForList .
+
+    Creation of shared memory for numpy arrays as described in
+    https://docs.python.org/3/library/multiprocessing.shared_memory.html#module-multiprocessing.shared_memory
 
     Parameters
     ----------
-    funktion : function
-        Function to process with arguments (looplist[i],args,kwargs)
-    qList : list
-        List of values for first argument in function.
-        qList value prepends the arguments args.
-    ncpu : int, optional
-        | number of cpus in the pool
-        | not given or 0   -> all cpus are used
-        | int>0      min (ncpu, mp.cpu_count)
-        | int<0      ncpu not to use
-    cb :function, optional
-        Callback after each calculation
-    debug : int
-        debug > 0 allows serial output for testing
+    ar : array
+        Numpy array to copy into a shared memory array.
 
     Returns
     -------
-    list : ndim function_return.ndim+1
-        The list elements will be prepended with the value qlist[i] as reference.
+        tuple with (SharedMemory, ar.shape, ar.dtype)
+
 
     Examples
     --------
     ::
 
-     def f(x,a,b,c,d):
-        return [x+a+b+c+d]
-     # loop over first argument here x
-     js.parallel.doForList(f,Qlist=np.arange(100),a=1,b=2,c=3,d=11)
+     import jscatter as js
+     import numpy as np
+
+     def model(q, ar):
+        # recover the array from shared memory
+        ar = js.formel.shared_recover(ar)
+
+        # here we do a difficult calculation for a single q value but the whole array
+        sum = np.sum(q*ar)
+        return sum
+
+     qlist=np.r_[0.1:10:0.1]
+
+     # example grid with about 1e6 points
+     cloud = js.ff.superball(1, 10, p=2, nGrid=100, returngrid=True).XYZ
+
+     # transfer large grid to shared memory
+     cloud_sh = js.formel.shared_create(cloud)
+
+     # do the calculation for a qlist values
+     res = js.formel.doForList(model, qlist, loopover='q', ar=cloud_sh)
+
+     # always free the shared memory (close and unlink)
+     js.formel.shared_close(cloud_sh)
+
+    Notes
+    -----
+    This function (with
+    :py:func:`~jscatter.parallel.shared_recover` and :py:func:`~jscatter.parallel.shared_close`) is more for experts.
+
+    The speedup e.g. in cloudScattering is not huge as the main time is consumed in calculations.
+
+    Also creating shared memory (~25ms for 1e6x3 array) takes around the same time as
+    numpy array pickling (~33ms) for transfer to a pool of workers.
+    Inside the pool the recover is much faster (~ 20µs) compared to unpickling (2.2.ms)
+
+    The speedup will be more important if e.g. the qlist (see example) is huge and the calculations are short.
+    In this case data transfer will be more important than the calculation.
+
 
     """
-    output = kwargs.pop('output', True)
-    ncpu = mp.cpu_count()
-    if 'ncpu' in kwargs:
-        if kwargs['ncpu'] < 0:
-            ncpu = max(ncpu + kwargs['ncpu'], 1)
-        elif kwargs['ncpu'] > 0:
-            ncpu = min(ncpu, kwargs['ncpu'])
-        del kwargs['ncpu']
-    cb = kwargs.pop('cb', None)
-    jobGetArg = kwargs.pop('jobGetArg', None)
-    if ('debug' in kwargs and kwargs['debug'] > 0) or ncpu == 1:  # in sequence for testing purposes
-        res = []
-        if 'debug' in kwargs:
-            del kwargs['debug']
-        if output: print('start NO pool  this is a test ')
-        for Q in qList:
-            # noinspection PyCallingNonCallable
-            res.append(funktion(*(Q,) + args, **kwargs))
-            if cb is not None:
-                cb(res[-1])
-    else:  # in parallel for production run
-        # print 'start pool of ',ncpu
-        pool = mp.Pool(ncpu)
-        jobs = [(Q, pool.apply_async(funktion, (Q,) + args, kwargs, callback=cb)) for Q in qList]
-        res = [job[1].get(jobGetArg) for job in jobs]
-        # clean up
-        pool.close()
-        pool.join()
-        # print 'closed pool again'
-    return res
+    assert isinstance(ar, np.ndarray), 'This method accepts only numpy arrays.'
+    assert mp.current_process().name == 'MainProcess', 'shared memory creation only in main process. '
+    # create shared memory
+    shm = shared_memory.SharedMemory(create=True, size=ar.nbytes)
+    # create array using the previous buffer
+    shared_ar = np.ndarray(ar.shape, dtype=ar.dtype, buffer=shm.buf)
+    # fill with data
+    shared_ar[:] = ar[:]
+    return shm, ar.shape, ar.dtype
+
+
+def shared_recover(shared):
+    """
+    Recover shared memory array inside a function evaluated in a pool of workers.
+
+    Parameters
+    ----------
+    shared : tuple created by shared_create
+
+    Returns
+    -------
+        array
+
+    """
+    # create array using the buffer
+    ar = np.ndarray(shared[1], dtype=shared[2], buffer=shared[0].buf)
+    return ar
+
+
+def shared_close(shared):
+    """
+    Close and unlink shared memory array.
+
+    Parameters
+    ----------
+    shared : tuple created by shared_create
+
+    """
+    # access buffer using its name
+    shared[0].close()
+    if mp.current_process().name == 'MainProcess':
+        shared[0].unlink()
+
+
+
