@@ -180,7 +180,7 @@ class lattice(object):
 
 
         """
-        pass
+        self._select = None
 
     def __getitem__(self, item):
         return self.points[item]
@@ -310,26 +310,36 @@ class lattice(object):
         grid._points contains all Nx[x,y,z,b]
 
         """
-        return self._points[np.abs(self._points[:, 3]) > _atol]
+        return self._points[self.nonzerob]
 
-    def prune(self, select):
+    @property
+    def nonzerob(self):
         """
-        Prune lattice to reduced number of points (in place)
+        Mask of nonzero b.
+        """
+        return np.abs(self._points[:, 3]) > _atol
+
+    def prune(self, select=None):
+        """
+        Prune lattice to reduced number of points (in place).
 
         Parameters
         ----------
-        select : bool array or function
-            A bool array to select points (shape N = len(grid.ball))
-            or a bool function that is evaluated for each [x,y,z,b]
+        select : bool array xN or function
+            A bool array of length grid points to keep only points that are true
+            or a bool function that is evaluated for each [x,y,z,b] evaluating to True.
+            By default all pounts with b=0 are removed.
 
         Examples
         --------
         ::
 
-         grid.prune(grid.ball>0)
-         grid.prune(lambda a:a[3]>0)
+         grid.prune(grid.ball>0)  # bool array
+         grid.prune(lambda a:a[3]>0)  # function
 
         """
+        if select is None:
+            select = self.nonzerob
         if callable(select):
             sel = np.array([select(point) for point in self._points])
             self._points = self._points[sel, :]
@@ -337,6 +347,9 @@ class lattice(object):
             # try if it was an bool array
             self._points = self._points[select, :]
 
+        # reset last selection
+        self._select = None
+
     def filter(self, funktion):
         """
         Set lattice points scattering length according to a function.
@@ -372,14 +385,18 @@ class lattice(object):
         # get float values
         v = np.array([funktion(point) for point in self._points[:, :3]])
         # set for v !=0, dont change others
-        choose = np.abs(v) > _atol
-        self._points[choose, 3] = v[choose]
+        select = np.abs(v) > _atol
+        self._points[select, 3] = v[select]
+        self._select = select
 
     def centerOfMass(self):
         """
         Center of mass as center of geometry.
         """
-        return self.points[:, :3].mean(axis=0)
+        if self.b.shape[0] > 0:
+            return self.points[:, :3].mean(axis=0)
+        else:
+            return [np.nan, np.nan, np.nan]
 
     def numberOfAtoms(self):
         """
@@ -443,11 +460,12 @@ class lattice(object):
         db1 = np.dot(np.array(v2), b1)
         dc = np.dot(self._points[:, :3] - corner, c1)
         dc1 = np.dot(np.array(v3), c1)
-        choose = (0 <= da) & (da <= da1) & (0 <= db) & (db <= db1) & (0 <= dc) & (dc <= dc1)
+        select = (0 <= da) & (da <= da1) & (0 <= db) & (db <= db1) & (0 <= dc) & (dc <= dc1)
         if invert:
-            self._points[~choose, 3] = b
+            self._points[~select, 3] = b
         else:
-            self._points[choose, 3] = b
+            self._points[select, 3] = b
+        self._select = select
 
     def planeSide(self, vector, center=None, b=1, invert=False):
         """
@@ -482,11 +500,12 @@ class lattice(object):
         v = v / la.norm(v)
         # np.dot is not thread safe
         dd = np.einsum('ij,j', self._points[:, :3] - c, v)
-        choose = (dd > 0)
+        select = (dd > 0)
         if invert:
-            self._points[~choose, 3] = b
+            self._points[~select, 3] = b
         else:
-            self._points[choose, 3] = b
+            self._points[select, 3] = b
+        self._select = select
 
     def inSphere(self, R, center=None, b=1, invert=False):
         """
@@ -528,11 +547,12 @@ class lattice(object):
         """
         if center is None:
             center = [0, 0, 0]
-        choose = la.norm(self._points[:, :3] - np.array(center), axis=1) < abs(R)
+        select = la.norm(self._points[:, :3] - np.array(center), axis=1) < abs(R)
         if invert:
-            self._points[~choose, 3] = b
+            self._points[~select, 3] = b
         else:
-            self._points[choose, 3] = b
+            self._points[select, 3] = b
+        self._select = select
 
     def inEllipsoid(self, abc, rotation=None, center=None, b=1, invert=False):
         """
@@ -580,11 +600,12 @@ class lattice(object):
             points=rotation.apply(points)
         # scale to normal form of ellipsoid and use norm<1
         scaledellipscoord = (points) / abc
-        choose = la.norm(scaledellipscoord, axis=1) < 1
+        select = la.norm(scaledellipscoord, axis=1) < 1
         if invert:
-            self._points[~choose, 3] = b
+            self._points[~select, 3] = b
         else:
-            self._points[choose, 3] = b
+            self._points[select, 3] = b
+        self._select = select
 
     def inCylinder(self, v, R, a=None, length=0, b=1, invert=False):
         """
@@ -634,22 +655,30 @@ class lattice(object):
         p = self._points[:, :3] - a
         # distance from a  (np.dot is not thread safe)
         pn = np.einsum('ij,j', p, n)
-        # choose if in distance from axis
-        choose = la.norm(p - (pn * n[:, None]).T, axis=1) < abs(R)
-        # not choose if outside cylinder length
+        # select if in distance from axis
+        select = la.norm(p - (pn * n[:, None]).T, axis=1) < abs(R)
+        # not select if outside cylinder length
         if length > 0:
-            choose[(pn < 0) | (pn > length)] = False
+            select[(pn < 0) | (pn > length)] = False
         elif length < 0:
-            choose[(pn > 0) | (pn < length)] = False
+            select[(pn > 0) | (pn < length)] = False
 
         if invert:
-            self._points[~choose, 3] = b
+            self._points[~select, 3] = b
         else:
-            self._points[choose, 3] = b
-        return
+            self._points[select, 3] = b
+        self._select = select
 
     alongLine = inCylinder
 
+    @property
+    def lastSelection(self):
+        """
+        Give last selection from filter, inSphere, inCylinder, inEllipsoid, planeSide, inParallelepiped.
+
+        """
+        return self._select
+
     def show(self, R=None, cmap='rainbow', fig=None, ax=None, atomsize=1):
         """
         Show the lattice in matplotlib with scattering length color coded.
@@ -667,8 +696,7 @@ class lattice(object):
         ax : Axes
             If given this axes is used for plotting.
         atomsize : float
-            Sphere size of the atoms with the smallest scattering length b.
-            Other sizes (radius) will be scaled according to b.
+            Sphere size of the atoms.
             Unfortunately matplotlib does not scale the point size when zooming.
 
         Returns
@@ -711,7 +739,9 @@ class lattice(object):
                 # r,c,i = ax.get_geometry()
                 ax.clear()
         # scatter plot of points
-        ax.scatter(points[:, 0], points[:, 1], points[:, 2], c=points[:, 3], s=points[:, 3] / bmin * atomsize ** 2,
+        scale = (points[:, 3] - bmin) / bmax
+        ax.scatter(points[:, 0], points[:, 1], points[:, 2], c=scale*100,
+                   s=atomsize,
                    cmap=cmap, vmin=bmin, vmax=bmax, depthshade=False)
 
         # try to plot unit cell with edge vectors
@@ -1170,9 +1200,9 @@ class rhombicLattice(lattice):
         # calc structure factor
         f2hkl = self._f2hkl(hkl)
         # selection rule
-        choose = (f2hkl > threshold * f2hkl.max())
+        select = (f2hkl > threshold * f2hkl.max())
 
-        return np.c_[bbb[choose], f2hkl[choose], hkl[choose]]
+        return np.c_[bbb[select], f2hkl[select], hkl[select]]
 
     def getRadialReciprocalLattice(self, size):
         """
